<?php
/**
 * 
 * @author Piotr Kozdęba <piotr.ko@modulesgarden.com>
 * 
 * Controls cart session
 */
class OPO_System_Sessionmanagement
{
    public static $dummydomain = 'thisistestdomainonlyandwontberegistered.test';
    
    
    /**
     * Reorganizing cart to leave only one product, eventually filtering also domains if parameters are supported
     * 
     * @param type $reorganizedomains       - true to reorganize domains with rest parameters data
     * @param type $domainstoleaveindomains - list of domains that should leave in domains list
     * @param type $productdomain           - main domain of the product
     * @param type $domainmode              - 'owndomain' if owndomain mode domain is selected; otherwise it will be treated like the user want to register new domain
     * @param type $mainisunavailable       - true if product main domain is unavailable
     */
    public static function reorganizeCart($reorganizedomains=false, $domainstoleaveindomains=array(), $productdomain='', $domainmode='register', $mainisunavailable=false) 
    {
        $products = array();
        $domains = array();

        if (isset($_SESSION['cart']))
        {
            $products = (isset($_SESSION['cart']['products']))?$_SESSION['cart']['products']:array();
            $domains = (isset($_SESSION['cart']['domains']))?$_SESSION['cart']['domains']:array();
        }
        
        $domain = '';
        
        $product_id = 0;
        
        $product_need_domain = true;
        
        if (!empty($products))
        {
            $latest_product = array_pop($products);
            if (count($products)>=1)
            {
                $last_product = array_pop($products);
                
                $latest_id = $latest_product['pid'];
                $db = new OPO_Database_Database();
                
                $available_addons = $db->query(
                        'SELECT `id` FROM `tbladdons`'.
                        ' WHERE `packages`='.$db->escape($latest_id).
                        ' OR `packages` LIKE '.$db->quote('%,'.$db->escapeNoQuotes($latest_id)).
                        ' OR `packages` LIKE '.$db->quote('%,'.$db->escapeNoQuotes($latest_id).',%').
                        ' OR `packages` LIKE '.$db->quote($db->escapeNoQuotes($latest_id).',%')
                        )->asArray('id','id');
                
                if ($available_addons && !empty($available_addons))
                {
                    if (!isset($last_product['addons']) || !is_array($last_product['addons']))
                        $last_product['addons'] = array();
                    
                    $new_product_addons = array_values(array_intersect($last_product['addons'], $available_addons));
                    
                    $latest_product['addons'] = $new_product_addons; 
                }
            }
            
            
            //only last selected product should always leave:
            $_SESSION['cart']['products'] = array($latest_product);
            
            //get product id:
            $product_id = $_SESSION['cart']['products'][0]['pid'];
            
            //checking if product require domain:
            $product_need_domain = OPO_WHMCS_Products::get($product_id, 'showdomainoptions')?true:false;

            //if product do not require domain than empty it:
            if (!$product_need_domain || ($reorganizedomains && $mainisunavailable && trim($domainmode)!='owndomain'))
            {
                $_SESSION['cart']['products'][0]['domain'] = '';
            }
            //saving correct productdomain to product if it is needed:
            elseif ($product_need_domain && $reorganizedomains && (!$mainisunavailable || trim($domainmode)=='owndomain'))
            {
                $_SESSION['cart']['products'][0]['domain'] = $productdomain;
            }
            
            //get product domain:
            $domain = $_SESSION['cart']['products'][0]['domain'];
        }
        
        $domain=trim($domain);
        
        if (!empty($domains) && $reorganizedomains)
        {
            //filtering domains to match last update:
            foreach ($domains as $k=>$domain_try)
            {
                //deleting domains not checked in the form:
                //(this are domains that should leave in cart/domains - it could be also main product domain)
                if (!in_array($domain_try['domain'],$domainstoleaveindomains))
                {
                    unset($domains[$k]);
                }
               
                //deleting main domain if this is owndomain mode:
                if (trim($domainmode)=='owndomain' && $domain && $domain==$domain_try['domain'])
                {
                    unset($domains[$k]);
                }
                
                //saving proper domains to session:
                $_SESSION['cart']['domains'] = array_values($domains);
                if($domainmode == 'transfer')
                {
                    $_SESSION['cart']['domains'][0]['type'] = 'transfer';
                }
            }
            
        }
        
    }
    
                
    
    /**
     * Returns main domain from cart - this is product domain or first of domains list if the product doesnt have one
     * If product has $dummydomain domain that it return empty string of course
     * 
     * @return string
     */
    public static function getCartDomain()
    {
        self::reorganizeCart();
        $products = array();
        $domains = array();
       
        if (isset($_SESSION['cart']))
        {
            $products = (isset($_SESSION['cart']['products']))?$_SESSION['cart']['products']:array();
            $domains = (isset($_SESSION['cart']['domains']))?$_SESSION['cart']['domains']:array();
        }
        
        $domain = '';
        $product_need_domain = true;
        
        //return if product need domain:
        if (!empty($products))
        {
            //get product domain:
            $domain = $_SESSION['cart']['products'][0]['domain'];
            
            //get product id:
            $product_id = $_SESSION['cart']['products'][0]['pid'];
            
            //check if product need domain:
            $product_need_domain = OPO_WHMCS_Products::get($product_id, 'showdomainoptions')?true:false;
        }
        
        //(product need domain)
        if ($product_need_domain)
        {
            if (trim($domain)==self::$dummydomain) $domain='';
            return $domain;
        }
        
        //return if product don't need domain:
        if (!empty($domains))
        {
            return $_SESSION['cart']['domains'][0]['domain'];
        }
        
        return '';
    }
    
    
    /**
     * Returns suggested domains from cart - this are not-product domains if product has domain or all without first on domains list if the product doesnt have one
     * If product has $dummydomain domain that it is considered to have domain of course
     * 
     * @return string
     */
    public static function getCartSuggestedDomains()
    {
        self::reorganizeCart();
        $products = array();
        $domains = array();
       
        if (isset($_SESSION['cart']))
        {
            $products = (isset($_SESSION['cart']['products']))?$_SESSION['cart']['products']:array();
            $domains = (isset($_SESSION['cart']['domains']))?$_SESSION['cart']['domains']:array();
        }
     
        //get product domain:
        $domain = '';
        
        if (!empty($products))
        {
            //get product domain:
            $domain = $_SESSION['cart']['products'][0]['domain'];
            
            //get product id:
            $product_id = $_SESSION['cart']['products'][0]['pid'];
            
            //check if product need domain:
            $product_need_domain = OPO_WHMCS_Products::get($product_id, 'showdomainoptions')?true:false;
        }
        
        $domain = trim($domain);
        
        //if product need domain
        if ($product_need_domain)
        {
            if (trim($domain)==self::$dummydomain) $domain='';
            foreach ($domains as $k=>$cartdomain)
            {
                if ($cartdomain['domain']===$domain)
                    unset ($domains[$k]);
            }
            return array_values($domains);
        }
        
        //return if product don't need domain:
        if (!empty($domains))
        {
            array_shift($domains);
            return $domains;
        }
        
        return array();
    }
    
    
    /**
     * Returns product id and main domain from cart
     * 
     * @return string
     */
    public static function getCartProductIdAndDomain()
    {
        //reorganizing is not needed here because it is already done by getCartDomain
        $maindomain = self::getCartDomain();
        
        $products = array();
        
        if (isset($_SESSION['cart']))
        {
            $products = (isset($_SESSION['cart']['products']))?$_SESSION['cart']['products']:array();
        }
        
        $product_id = 0;
        
        //return if product need domain:
        if (!empty($products))
        {
            //get product id:
            $product_id = $_SESSION['cart']['products'][0]['pid'];
        }
        
        return array($product_id, $maindomain);
    }
    
    
}